
#load'C:\Users\Keith\Documents\ShopProjects\TestPlugins\KK_MortiseTool.rb'

  #portions of this code are modifications of examples from Google
  # Copyright 2005-2007, Google, Inc.
  # V2.0
  # Name :          KK_MortiseTool  ###
  # Date :	     4 20 2011
  # Description : Creates complete joints for Mortise & Tenon Joints or Loose Tenon Joints
  #               Also will draw a single Tenon or Mortise
  #               Tenons or Mortises can have Square or Radised Ends
  #               Also will draw Haunched Tenons and their mortise
  #               Will draw dowell holes for doweled joint. Use LT_Mortise and select "Dowel" for End Shape
  #               Mortise depth in second part is specified separate so it can be different than the Tenon Length
  #               Added capability of Input Forms to accept equations, fractions &/or numbers
  #               Tool is added to the "Draw" menu
  #               See SU drawing MortiseToolDim.skp   

  # Author :       Keith Krueger (with a big help from plugin forum:)
  #
 
 require 'sketchup.rb'
  



#creates dado in component face
 class MortiseTool #*****************************************************************
   #@status=0 start,  =1 select component, =2 first edge, =3 second edge
   #Selection problem handler ops, 0 ok,  1 not point/comp/edge, 2 edge1 not valid, 3 edge2 not valid
   
   @@todist=nil   #make class variables persist
   @@edge=nil
   @@depth=nil  
   @@width=nil
   @@ed2toend=nil
   @@ed1toend=nil
   @@hlength=nil   
   @@haunch=nil
   
  def initialize
   @entity=nil
   @ss=nil
   @status=-1
   @inpt=nil
   @tinpt=nil
   @ip=nil
   @component=1
   @first_comp=nil
   @second_comp=nil
   @tform=nil
   @tform2=nil
   @joint=1
   @hedge1=nil
   @hedge2=nil
   
   #defaults data for input forms
   if !@@todist then @@todist=0.25.inch end           
   if !@@edge then @@edge="Square" end
   if !@@depth then @@depth=0.875.inch end                  
   if !@@width then @@width=0.25.inch end   
   if !@@ed2toend then @@ed2toend=0.5.inch end #Shoulder2 dim
   if !@@ed1toend then @@ed1toend=0.25.inch end #Shoulder1 dim
   if !@@hlength then @@hlength=0.25.inch end #Haunched Tenon Length
   if !@@haunch then @@haunch="Shoulder 1" end 
   
  end #def initialize

  #is called when first activated
  def activate
   @inpt = Sketchup::InputPoint.new           #set up Input point and temp and diplayed
   @tinpt= Sketchup::InputPoint.new
   @ip= Sketchup::InputPoint.new
   
   case @@tooltype
     when "Tenon"
       @statusstring="Select Component for Tenon, or [Space Bar] to EXIT"
       @status=1
     when "MT_Tenon"
       @statusstring="Select First Component for MT-Joint, or [Space Bar] to EXIT"
       @status=0
     when "LT_Mortise"
       @statusstring="Select First Component for LT-Joint, or [Space Bar] to EXIT"
       @status=0
     when "Mortise"
       @statusstring="Select Component for Mortise, or [Space Bar] to EXIT"
       @status=1
   end # case @@tooltype   
   #@displayed = false

   #lets clear any componenent that is already selected
   model=Sketchup.active_model
   view = Sketchup.active_model.active_view
   @ss=model.selection
   le=@ss.count
   Sketchup::set_status_text @statusstring,SB_PROMPT
   if le>=1 then @ss.clear end #Clear Selection
   self.reset(nil)   
  end #def activate
  
  #is called if tool is deactivated because a different tool is selected
  def deactivate(view)
   view.invalidate 
  end
  
  def resume(view)
   view.invalidate
   Sketchup::set_status_text @statusstring,SB_PROMPT
   
  end

  #onCancel is called when user hits escape etc
  def onCancel(flag, view)
    #Debug UI.messagebox("Cancel, flag = " + flag.to_s)
    #if (flag==0)
      view.invalidate
      @oops=0
      @ss.clear
      @component=1
      @first_comp=nil
      @second_comp=nil
      if @first_comp!=nil then @first_comp.hidden=false end
      if @second_comp!=nil then @second_comp.hidden=false end
      if @hedge1!=nil then @hedge1.hidden=false end
      if @hedge2!=nil then @hedge2.hidden=false end
      case @@tooltype
        when "Tenon"
          @statusstring="Select Component for Tenon, or [Space Bar] to EXIT"
          @status=1             
        when "MT_Tenon"
          @statusstring="Select First Component for MT-Joint, or [Space Bar] to EXIT"
          @status=0             
        when "LT_Mortise"
          @statusstring="Select First Component for LT-Joint, or [Space Bar] to EXIT"
          @status=0             
        when "Mortise"
          @statusstring="Select Component for Mortise, or [Space Bar] to EXIT"
          @status=1             
      end # case @@tooltype
      Sketchup::set_status_text @statusstring,SB_PROMPT
    #else
      #view.invalidate
    #end
  end

#Translate a string to a float value, accepting mathematical forumal in the string - return nil of invalid
def equationString(s)
  #Debug UI.messagebox("testNo = " + s)
	return nil unless s
	s = s.strip.upcase
	lform = ["PI", "sin", "cos", "tan", "atan2", "sqrt", "log", "log10", "exp", "ceil", "floor", "round"]
	lform.each_with_index { |f, i| s = s.gsub(f.upcase, "_#{i}_") }
	s = s.gsub(/,\d/) { |c| '.' + c[1..1] }
	return nil if s =~ /[^\s\+\-\*\.\,eE_\%\/\%\(\)\d]/
	s = s.gsub(/(\A\.)\d+/) { |ss| '0' + ss }
	s = s.gsub(/(\s|\(|\+|\-)(\.)(\d+)/) { |ss| $1 + '0.' + $3 }
	s = s.gsub(',', '.').gsub(/(\d+)\.*(\d*)(e|E)?(\d*)/) { |ss| (ss =~ /\.|e|E/) ? ss : ss + '.0' }
	s = s.gsub(/_(\d+)\.\d_/) { |f| "Math::#{lform[$1.to_i]}" }
	s = s.gsub(/\s+\(/, '(')	
	begin 
		eval "0.0 + (#{s})"
	rescue Exception => detail
		nil
	end	
end

 #reset to start over again
  def reset(view)
   UI.Messagebox"def reset"
   if @@tooltype=="Tenon" || @@tooltype=="Mortise" then @status=1  end
   if @@tooltype=="MT_Tenon" || @@tooltype=="LT_Mortise" then @status=0  end
   #@displayed=false
   @entity=nil
   @first_comp=nil
   @second_comp=nil
   Sketchup::set_status_text @statusstring,SB_PROMPT
  end

   #cruise for a point in component
  def onMouseMove(flags, x, y, view)
   Sketchup::set_status_text @statusstring,SB_PROMPT
   if (@status==2 or @status==3)
    @tinpt.pick view, x, y
    if (@tinpt.valid?) 
      changed=@tinpt != @inpt
      @inpt.copy! @tinpt 
      #view.tooltip = @tinpt.tooltip
      #display point
      if ( changed and (@inpt.display? or @displayed))
       view.invalidate 
      end
    end
   end
  end #def onMouseMove

  #get a point
  def onLButtonDown(flags, x, y, view)
     #Debug UI.messagebox("@status = " + @status.to_s )
     #Debug UI.messagebox("@first_comp = " + @first_comp.to_s )
     #Debug UI.messagebox("@@tooltype = " + @@tooltype )
     if(@first_comp==nil)&& ((@@tooltype=="MT_Tenon") || (@@tooltype=="LT_Mortise"))
       #Debug            UI.messagebox("Pick first component" )
       ph=view.pick_helper
       ph.do_pick x,y
       @entity=ph.best_picked
       @oops=0
       if @entity.typename=="ComponentInstance"
         @first_comp=@entity
         @ss.add(@entity)
         #Debug UI.messagebox("First Component Selected = " + @entity.to_s)
         case @@tooltype                    
           when "MT_Tenon"
             @statusstring="Select Second Component for MT-Joint, or [Space Bar] to EXIT"                       
           when "LT_Mortise"
             @statusstring="Select Second Component for LT-Joint, or [Space Bar] to EXIT"                    
         end # case @@tooltype
         Sketchup::set_status_text @statusstring,SB_PROMPT
       else
         UI.messagebox "Selection of first component invalid"
         @first_comp=nil
         @status=0
       end #if entity.typename=="ComponentInstance"
     elsif(@second_comp==nil)&& ((@@tooltype=="MT_Tenon") || (@@tooltype=="LT_Mortise"))
       #Debug            UI.messagebox("Pick Second component" )
       ph=view.pick_helper
       ph.do_pick x,y
       @entity=ph.best_picked
       @oops=0
       if @entity.typename=="ComponentInstance"
         @second_comp=@entity
         @ss.add(@entity)
         #Debug UI.messagebox("Second Component Selected = " + @entity.to_s)
         @ss.clear
         @status=2
         @entity=@first_comp
         @ss.add(@first_comp)
         @second_comp.hidden=true          
          if (@@tooltype=="Tenon") || (@@tooltype=="MT_Tenon")
            @statusstring="Select First Edge in component Parallel to Tenon Width"
            Sketchup::set_status_text @statusstring,SB_PROMPT
          else
            @statusstring="Select First Edge in component Parallel to Mortise Length"
            Sketchup::set_status_text @statusstring,SB_PROMPT
          end
       else
         UI.messagebox "Selection of Second component invalid"
         @second_comp=nil
         @status=0
       end #if entity.typename=="ComponentInstance"

     elsif(@status==1) #1 select component*************
      ph=view.pick_helper
      ph.do_pick x,y
      @entity=ph.best_picked
      @oops=1
      if @entity #2
        type=@entity.typename
        if type=="ComponentInstance" #3
          model=Sketchup.active_model
          #@ss=model.selection
          @ss.add(@entity)
          
          if (@@tooltype=="Tenon") || (@@tooltype=="MT_Tenon")
            @statusstring="Select First Edge in component Parallel to Tenon Width"
            Sketchup::set_status_text @statusstring,SB_PROMPT
          else
            @statusstring="Select First Edge in component Parallel to Mortise Length"
            Sketchup::set_status_text @statusstring,SB_PROMPT
          end
          @status=2
          @oops=0        
        end #3
      end #2
     elsif(@status==2)#1
	    @ip.pick view,x,y
	    @oops=1
      if(@ip.valid?)#4
	      if(@ip.edge) #5
	        @oops=2
	        if (@entity.definition==@ip.edge.parent) #6
	          if ((!@ip.vertex) and (@ip.depth>0))#7
	                  
              #This secton added to get point transformations
              if @component==1 then @tform = @ip.transformation  end
              if @component==2 then @tform2 = @ip.transformation.inverse end
              #Debug if @component==1 then UI.messagebox("@tform = " + @tform.to_s) end
              #Debug if @component==2 then UI.messagebox("@tform2 = " + @tform2.to_s) end
        
	          	@oops=0
		          @edge1=@ip.edge
              @statusstring="Select Second Edge in component Parallel to First Edge"
              Sketchup::set_status_text @statusstring,SB_PROMPT		          		          
              @status=3
	         	end #7
	        end	#6
	      end #5	      
	    end #4
     elsif @status==3 #1
	    @ip.pick view,x,y
      @oops=1
      if(@ip.valid?) #9
        if(@ip.edge)#10
          @oops=3
          if (@entity.definition==@ip.edge.parent)#11
	          if ((!@ip.vertex) and !(@ip.edge==@edge1) and (@ip.depth>0)) #12
		          @oops=0
		          Sketchup.active_model.start_operation "undo"
		          @edge2=@ip.edge		        		        		        		        		        
		          #collect entities
              componentdefinition=@entity.definition
              allent=componentdefinition.entities
              codef=@entity.definition
              #collect face entities
              faceent=[]
              allent.each do |ent|
              if ent.typename=="Face" #13
                faceent << ent
              end #13
            end #12
		        #find face that has edges 1 and 2
		    	  face=@edge1.common_face @edge2
		    	  if (face==nil)#14                    #get valid face
		          @oops=3
            else #14
              #Debug UI.messagebox"got valid face######################"
			        #get points in local coord system for face
			        #xaxis parallel to @edge1
			        orig=@edge1.start
              x,y,z=face.normal.axes
			        #get vector parallel to @edge1
			        p1=@edge1.start.position
			        p2=@edge1.end.position			          
			        #Debug UI.messagebox("Start Position = " + p1.to_s)
			        #Debug UI.messagebox("End Position = " + p2.to_s)
			        if @component==1 			          
			          ar=[p2.x-p1.x,p2.y-p1.y,p2.z-p1.z]
			          xx=ar.normalize
		            yy=z.cross xx
                localt=Geom::Transformation.axes(orig,xx,yy,z).invert!
			          backt=Geom::Transformation.axes(orig,xx,yy,z)
			          edges=face.outer_loop.edges
			          eind=edges.index(@edge1)
			          if @@width<0.001 #15 
			            #Reset width to pos no!!!
                  @@width=-1*@@width
                end #15
                #Select prompt based on which tool was called Mortise or Tenon
                if @@edge=="Haunched" then @@edge="Square" end
                if (@@tooltype=="Tenon") || (@@tooltype=="MT_Tenon") #16
			            prompts = ["Edge1 to Tenon","End Shape", "Thickness","Length","End1 Shoulder","End2 Shoulder"]

                  defaults= [@@todist.to_s,@@edge,@@width.to_s,@@depth.to_s,@@ed1toend.to_s,@@ed2toend.to_s]
               
                  choices = ["","Square|Round|Haunched","",""]
   
                  results = inputbox prompts, defaults, choices, "Add Tenon"
                else #16
         
			            prompts = ["Edge1 to Mortise","End Shape", "Width","Depth","End1 Shoulder","End2 Shoulder"]

                  defaults= [@@todist.to_s,@@edge,@@width.to_s,@@depth.to_s,@@ed1toend.to_s,@@ed2toend.to_s]
               
                  choices = ["","Square|Round|Dowel","",""]
   
                  results = inputbox prompts, defaults, choices, "Add Mortise"
                end  #16
                #return if not results
			          if not results  #17 user cancel
			            @status=2
			            @edge1=nil
			            @edge2=nil
			            return
			          end #17
                			          
                @@todist=equationString(results[0])
                if @@todist== nil then @@todist=results[0] end
                @@todist=@@todist.to_f #Make sure its a float no
                                
                @@edge=results[1]
                                
                @@width=equationString(results[2])
                if @@width== nil then @@width=results[2] end
                @@width=@@width.to_f #Make sure its a float no
                
                @@depth=equationString(results[3])
                if @@depth== nil then @@depth=results[3] end
                @@depth=@@depth.to_f #Make sure its a float no
                
                @@ed1toend=equationString(results[4])
                if @@ed1toend== nil then @@ed1toend=results[4] end
                @@ed1toend=@@ed1toend.to_f #Make sure its a float no
                
                @@ed2toend=equationString(results[5])
                if @@ed2toend== nil then @@ed2toend=results[5] end
                @@ed2toend=@@ed2toend.to_f #Make sure its a float no 
                
                #Input Data Form for Haunched Tenon
                if @@edge=="Haunched"
                  prompts = ["Haunch Length","End Shape"]
                  defaults= [@@hlength.to_s,@@haunch]               
                  choices = ["","Shoulder 1|Shoulder 2|Both","",""]   
                  results = inputbox prompts, defaults, choices, "Haunched Tenon Dim"
                  
                  #Get Data from Input Form
                  @@hlength=equationString(results[0])
                  if @@hlength== nil then @@hlength=results[0] end
                  @@hlength=@@hlength.to_f #Make sure its a float no

                  @@haunch=results[1]
                  #Debug UI.messagebox("Haunched Tenon Length = "+@@hlength.to_s)
                  #Debug UI.messagebox("Haunched Shoulder = "+@@haunch)
                end #if @@edge=="Haunched"  
           
			          if @@edge=="Square" || @@edge=="Haunched" #18
			            ofstov=@@todist+@@width
			            r=0.0
			          else #18
                  #@@edge=="Round" || @@edge=="Dowel"
			            ofstov=@@todist+@@width
			            r=0.5*@@width
			            #@@ed2toend=@@ed2toend-(2*r)
			            #57.29578
			            dely30=r*0.866
			            delx30=r*0.5
			            dely60=r*0.5
			            delx60=r*0.866
			          end #18
			          ofstto=ofstov-@@width
			          p3=@edge2.start.position.transform(localt)
			          en=eind-edges.length+1;i=eind-1
			          dir=p3.y<=>0 
			          ofstov=dir*ofstov
			          ofstto=dir*ofstto
			          frstov=0
			          frstto=0
			          while i>=en
			            p1=edges[i].start.position.transform(localt)
                  p2=edges[i].end.position.transform(localt)
                  p1y=p1.y;p2y=p2.y
				          if p1y>p2y then p1y,p2y=p2y,p1y end #single line
                  if (p1y..p2y)===ofstov #19
				            x=p1.x+(ofstov-p1.y)/(p2.y-p1.y)*(p2.x-p1.x)
				            if frstov==0 then x1=x;frstov=1 end #single line
				            x2=x
                  end	#19
			     	      if ofstto.abs>0.001 #20
				            if (p1y..p2y)===ofstto #21
				              x=p1.x+(ofstto-p1.y)/(p2.y-p1.y)*(p2.x-p1.x)
				              if frstto==0 then x4=x;frstto=1 end #single line
				              x3=x
				            end #21
				          end #20
			            i-=1
			          end #while
                #This IF statement always puts the mortise inside the board end
                #Debug UI.messagebox("End1 Shoulder = " + @@ed1toend.to_s)
			          #Debug UI.messagebox("End2 Shoulder = " + @@ed2toend.to_s)
			          #Debug UI.messagebox("Radius = " + r.to_s)                
			          if x1>x2 #22
			            delx_dir=1
			            dely_dir=1
			            x1h=x1  #Add points for Haunched Tenon Joint
			            x2h=x2
			            x1=x1-(@@ed1toend+r)
			            x2=x2+(@@ed2toend+r)
			            if @@edge=="Dowel" then x2=x1 end # Shoulder 2 Dim ignored for round hole
			            if x1<=x2 then x1=x1+0.002 end  # Add 0.002 to make round hole if equal results unpredictable
			            x3=x2
			            x4=x1		     
			          else #22
			            delx_dir=2
			            x1h=x1  #Add points for Haunched Tenon Joint
			            x2h=x2
			            x1=x1+(@@ed1toend+r)
			            x2=x2-(@@ed2toend+r)
			            if @@edge=="Dowel" then x2=x1 end # Shoulder 2 Dim ignored for round hole
			            if x2<=x1 then x2=x2+0.002 end # Add 0.002 to make round hole if equal results unpredictable
			            x3=x2
			            x4=x1
			          end #22
			          x3h=x2h
			          x4h=x1h
			          pt1=Geom::Point3d.new(x1,ofstov,0)
			          pt2=Geom::Point3d.new(x2,ofstov,0)		   
			          pt4=Geom::Point3d.new(x4,ofstto,0)
			          pt3=Geom::Point3d.new(x3,ofstto,0)
			          
			          po1=pt1.transform(backt)
			          po4=pt4.transform(backt)
			          po2=pt2.transform(backt)
			          po3=pt3.transform(backt)
			          
			          #Add points for Haunched Tenon Joint
			          hd=@@hlength
			          pt1h0=Geom::Point3d.new(x1h,ofstov,0)
			          pt2h0=Geom::Point3d.new(x2h,ofstov,0)		   
			          pt4h0=Geom::Point3d.new(x4h,ofstto,0)
			          pt3h0=Geom::Point3d.new(x3h,ofstto,0)
			          pt1hd=Geom::Point3d.new(x1,ofstov,hd)
			          pt2hd=Geom::Point3d.new(x2,ofstov,hd)		   
			          pt4hd=Geom::Point3d.new(x4,ofstto,hd)
			          pt3hd=Geom::Point3d.new(x3,ofstto,hd)
			          po1hd=pt1hd.transform(backt)
			          po4hd=pt4hd.transform(backt)
			          po2hd=pt2hd.transform(backt)
			          po3hd=pt3hd.transform(backt)
			          po1h=pt1h0.transform(backt)
			          po4h=pt4h0.transform(backt)
			          po2h=pt2h0.transform(backt)
			          po3h=pt3h0.transform(backt)
			          #Debug UI.messagebox("Points for haunched ends po1h...4 " + po1h.to_s + po2h.to_s + po3h.to_s + po4h.to_s)
                #Debug UI.messagebox("Points for haunched ends po1hd...4hd " + po1hd.to_s + po2hd.to_s + po3hd.to_s + po4hd.to_s)
                #Debug UI.messagebox("Points for haunched ends po1...4 " + po1.to_s + po2.to_s + po3.to_s + po4.to_s)
			   			   
			          			   
			          if @@edge=="Round" || @@edge=="Dowel"   #23 Points for top and bottom arc
			            if delx_dir==2 #24
			              dely_dir=-1
			              delx30=-1*delx30
			              delx60=-1*delx60
			              r=-1*r
			              @@width=-1*@@width
			            end #24
			           #Debug UI.messagebox "Start calculating pt2a to pt4e"			     
			           pt2a=Geom::Point3d.new(x2-delx30,ofstov+(@@width/2)-(dely30*dely_dir),0)
			           pt2b=Geom::Point3d.new(x2-delx60,ofstov+(@@width/2)-(dely60*dely_dir),0)
			           pt2c=Geom::Point3d.new(x2-r,ofstov+(@@width/2),0)
			           pt2d=Geom::Point3d.new(x2-delx60,ofstov+(@@width/2)+(dely60*dely_dir),0)
			           pt2e=Geom::Point3d.new(x2-delx30,ofstov+(@@width/2)+(dely30*dely_dir),0)
			       
			           pt4a=Geom::Point3d.new(x4+delx30,ofstto-(@@width/2)+(dely30*dely_dir),0)
			           pt4b=Geom::Point3d.new(x4+delx60,ofstto-(@@width/2)+(dely60*dely_dir),0)
			           pt4c=Geom::Point3d.new(x4+r,ofstto-(@@width/2),0)
			           pt4d=Geom::Point3d.new(x4+delx60,ofstto-(@@width/2)-(dely60*dely_dir),0)
			           pt4e=Geom::Point3d.new(x4+delx30,ofstto-(@@width/2)-(dely30*dely_dir),0)
			           #Debug UI.messagebox "End calculating pt2a to pt4e"
			     
                 po2a=pt2a.transform(backt)
                 po2b=pt2b.transform(backt)
                 po2c=pt2c.transform(backt)
                 po2d=pt2d.transform(backt)
                 po2e=pt2e.transform(backt)
                 po4a=pt4a.transform(backt)
                 po4b=pt4b.transform(backt)
                 po4c=pt4c.transform(backt)
                 po4d=pt4d.transform(backt)
                 po4e=pt4e.transform(backt)
                 #Debug UI.messagebox "Completed Transforming  pt2a to pt4e"                 
               end #23
                  #Debug UI.messagebox("@tform = " + @tform.to_s)
                  #Debug UI.messagebox("po1 = " + po1.to_s)               
                  #Debug UI.messagebox "#This section uses the point transformation to save points in Global Coords"

                 @pg1=po1.transform(@tform)
			           @pg4=po4.transform(@tform)
			           @pg2=po2.transform(@tform)
			           @pg3=po3.transform(@tform)
			           @pg1hd=po1hd.transform(@tform)
			           @pg4hd=po4hd.transform(@tform)
			           @pg2hd=po2hd.transform(@tform)
			           @pg3hd=po3hd.transform(@tform)
			           @pg1h=po1h.transform(@tform)
			           @pg4h=po4h.transform(@tform)
			           @pg2h=po2h.transform(@tform)
			           @pg3h=po3h.transform(@tform)
			           
			           if @@edge=="Round" || @@edge=="Dowel"
                   @pg2a=po2a.transform(@tform)
                   @pg2b=po2b.transform(@tform)
                   @pg2c=po2c.transform(@tform)
                   @pg2d=po2d.transform(@tform)
                   @pg2e=po2e.transform(@tform)
                   @pg4a=po4a.transform(@tform)
                   @pg4b=po4b.transform(@tform)
                   @pg4c=po4c.transform(@tform)
                   @pg4d=po4d.transform(@tform)
                   @pg4e=po4e.transform(@tform)
                 end #if @@edge=="Round"
                 #Debug UI.messagebox("po1 Global Coors = " + @pg1.to_s)  
              end #if @component==1
              if @component==2
                #Debug UI.messagebox"#This section transforms Global points saved from Comp1 to Local Comp2"
                po1=@pg1.transform(@tform2)
			          po4=@pg4.transform(@tform2)
			          po2=@pg2.transform(@tform2)
			          po3=@pg3.transform(@tform2)
			          po1hd=@pg1hd.transform(@tform2)
			          po4hd=@pg4hd.transform(@tform2)
			          po2hd=@pg2hd.transform(@tform2)
			          po3hd=@pg3hd.transform(@tform2)
			          po1h=@pg1h.transform(@tform2)
			          po4h=@pg4h.transform(@tform2)
			          po2h=@pg2h.transform(@tform2)
			          po3h=@pg3h.transform(@tform2)
			          if @@edge=="Round" || @@edge=="Dowel"
                  po2a=@pg2a.transform(@tform2)
                  po2b=@pg2b.transform(@tform2)
                  po2c=@pg2c.transform(@tform2)
                  po2d=@pg2d.transform(@tform2)
                  po2e=@pg2e.transform(@tform2)
                  po4a=@pg4a.transform(@tform2)
                  po4b=@pg4b.transform(@tform2)
                  po4c=@pg4c.transform(@tform2)
                  po4d=@pg4d.transform(@tform2)
                  po4e=@pg4e.transform(@tform2)
                end #if @@edge=="Round"
                  #Debug UI.messagebox("po1 Local Coors = " + po1.to_s) 
              end #if @comp==2 			   
			          #Debug UI.messagebox "end if @component==2"
			          if @@edge=="Round" || @@edge=="Dowel" #25
			            #Debug UI.messagebox "Add cface for round starts here"
			            #Debug UI.messagebox("Points for round ends are" + po1.to_s + po2.to_s + po2a.to_s + po2b.to_s+ po2c.to_s+ po2d.to_s+ po2e.to_s+ po3.to_s+ po4.to_s+ po4a.to_s+ po4b.to_s+ po4c.to_s+ po4d.to_s+ po4e.to_s)
			            #if @component==1 then @@ed2toend=@@ed2toend+(2*r) end #Resets Length to origional Data			             
			            cface=codef.entities.add_face po1,po2,po2a,po2b,po2c,po2d,po2e,po3,po4,po4a,po4b,po4c,po4d,po4e
			            #Debug UI.messagebox "Add cface for round ends here"			            
			          else #25
                  #Debug UI.messagebox("Global Points for Haunched ends are @pg1hd,@pg2hd,@pg3hd,@pg4hd " + @pg1hd.to_s + @pg2hd.to_s + @pg3hd.to_s + @pg4hd.to_s) 
                  #Debug UI.messagebox("Points for square ends are po1,po2,po3,po4 " + po1.to_s + po2.to_s + po3.to_s + po4.to_s)
                  #For Haunched Tenon pull haunch part of tenon first
                  if @@edge=="Haunched"                    
                    #Debug UI.messagebox"Haunched joint detail here"
                    case @@haunch
                      when  "Shoulder 1"
                        #Debug UI.messagebox"Haunched Shoulder 1"
                        #Debug UI.messagebox("Points for haunched ends are po1,po2,po3,po4 " + po1h.to_s + po2.to_s + po3.to_s + po4h.to_s)
                        cface=codef.entities.add_face po1h,po2,po3,po4h
                        if @component==1 then hd=@@hlength end
                        if @component==2 then hd=-1*@@hlength end
                        cface.pushpull hd
                      when  "Shoulder 2"
                        #Debug UI.messagebox"Haunched Shoulder 2"
                        #Debug UI.messagebox("Points for haunched ends are po1,po2,po3,po4 " + po1.to_s + po2h.to_s + po3h.to_s + po4.to_s)
                        cface=codef.entities.add_face po1,po2h,po3h,po4
                        if @component==1 then hd=@@hlength end
                        if @component==2 then hd=-1*@@hlength end
                        cface.pushpull hd
                      when  "Both"                        
                        #Debug UI.messagebox("Points for haunched ends are po1,po2,po3,po4 " +  po1h.to_s + po2h.to_s + po3h.to_s + po4h.to_s)                        
                        cface=codef.entities.add_face po1h,po4h,po3h,po2h
                        if @component==1 then hd=@@hlength end
                        if @component==2 then hd=-1*@@hlength end
                        #Debug UI.messagebox("Haunched Both Length = " + hd.to_s)
                        cface.pushpull hd
                    end # case @@haunch                    
                    po1=po1hd
                    po2=po2hd
                    po3=po3hd
                    po4=po4hd
                  end #Haunched 
			            cface=codef.entities.add_face po1,po4,po3,po2
                  #Debug UI.messagebox"Add cface for square ends here"
                  #Add Tenon/Mortise for Haunched Tenon Joint                                    		            
			       end #25
			       if @component==1  #26
			       #Debug UI.messagebox("ToolType = "  + @@tooltype) 
			         if (@@tooltype=="Tenon") || (@@tooltype=="MT_Tenon")  then h=@@depth  end
			         if @@edge=="Haunched" then h=(h-@@hlength) end
               if (@@tooltype=="Mortise") || (@@tooltype=="LT_Mortise") then  h=-1*@@depth end
             else#26 
		           prompts = ["Component 2 Mortise Depth"]
               defaults= [@@depth.to_s]                  
               results = inputbox prompts, defaults, "Mortise Depth for Component 2 Complete Joint"
               
               h=equationString(results[0])
               if h== nil then h=results[0] end
               h=h.to_f #Make sure its a float no

               if @@edge=="Haunched" then h=(h-@@hlength) end
               h=-1*h #Change to negative to push in
               #if (@@tooltype=="MT_Tenon")  then h=-1*h  end
               #if (@@tooltype=="LT_Mortise") then  h=-1*h end
             end #26
             	   
             cface.pushpull h
             Sketchup.active_model.commit_operation
             #Debug UI.messagebox("End of Operation, status = " + @status.to_s)
         
             #This section is New to add Mortise        
			       #This Tenon is Complete ask user for edge to start again
			       if @component==1
			         case @@tooltype                    
                 when "MT_Tenon"
                   @statusstring="Select Second Component to Complete MT-Joint, or [Space Bar] to EXIT"                       
                 when "LT_Mortise"
                   @statusstring="Select Second Component to Complete LT-Joint, or [Space Bar] to EXIT"
                 when "Tenon"
                   @statusstring="Tenon Complete;  "
                   Sketchup::set_status_text @statusstring,SB_PROMPT
                   result=UI.messagebox "Tenon Complete; Start New Tenon? ", MB_YESNO
                   if result==7 then Sketchup.send_action("selectSelectionTool:") end
                   if result==6 then onCancel(result, view) end
                 when "Mortise"
                   @statusstring="Mortise Complete;  "
                   Sketchup::set_status_text @statusstring,SB_PROMPT
                   result=UI.messagebox "Mortise Complete; Start New Mortise? ", MB_YESNO
                   if result==7 then Sketchup.send_action("selectSelectionTool:") end
                   if result==6 then onCancel(result, view) end                    
               end # case @@tooltype
               @status=1             
               @ss.clear
               @oops=0
               @component=2
               if @first_comp!=nil then @first_comp.hidden=true end
               if @second_comp!=nil then @second_comp.hidden=false end
               if @hedge1!=nil then @hedge1.hidden=false end
               if @hedge2!=nil then @hedge2.hidden=false end
             else
               if @first_comp!=nil then @first_comp.hidden=false end
               if @second_comp!=nil then @second_comp.hidden=false end
               if @hedge1!=nil then @hedge1.hidden=false end
               if @hedge2!=nil then @hedge2.hidden=false end
               @statusstring="Joint Complete;  "
               Sketchup::set_status_text @statusstring,SB_PROMPT
               result=UI.messagebox "Joint Complete; Start New Joint? ", MB_YESNO
               if result==7 then Sketchup.send_action("selectSelectionTool:") end
               if result==6 then onCancel(result, view) end
             end #if @component==1
             #Debug UI.messagebox("End New Section, status = " + @status.to_s)             	
            end #14
            #Debug UI.messagebox("End 14, status = " + @status.to_s)	                                ####################################
           end #11
           #Debug UI.messagebox("End 11, status = " + @status.to_s)	 
         end #10		
        end #9
        #Debug UI.messagebox("End 9, status = " + @status.to_s) 
      end #1
	    #Debug UI.messagebox("End 1, status = " + @status.to_s)   
      #Debug UI.messagebox("End 27, status = " + @status.to_s)	  
     end #if status & elseif status
     #Selection problem handler oops, 0 ok,  1 not point/comp/edge, 2 edge1 not valid, 3 edge2 not valid
     case @oops
       when 1
          UI.messagebox "Invalid point"
       when 2
          @hedge1=@ip.edge
	        @hedge1.hidden = true
          UI.messagebox "Edge 1 blocked: Pick again"
          @edge1=nil
          @status=2
       when 3
          @hedge2=@ip.edge
          @hedge2.hidden = true
          UI.messagebox "Edge 2 blocked: Pick again"
          @edge2=nil
          @status=3  
	     end # case @oops     
     #Debug UI.messagebox("End A, status = " + @status.to_s)
  end # def OnLButtonDown
  #Debug UI.messagebox("End B, status = " + @status.to_s)  

  #draw is called when we want to display then point
  def draw(view)
   if( @inpt.valid? and @inpt.display? )
    @inpt.draw view
    @displayed=true
   else
    @displayed=false
   end
  end

  
 end #end Class MortiseTool*********************************************

 #function acts as short cut for selecting tool
 def addtenon
   @@tooltype="Tenon"
   Sketchup.active_model.select_tool MortiseTool.new
 end

 #function acts as short cut for selecting tool
 def addmortise
   @@tooltype="Mortise"
   Sketchup.active_model.select_tool MortiseTool.new
 end 
 
 #function acts as short cut for selecting tool
 def mortiseboth
   @@tooltype="LT_Mortise"
   Sketchup.active_model.select_tool MortiseTool.new
 end
 
  def tenonandmortise
   @@tooltype="MT_Tenon"
   Sketchup.active_model.select_tool MortiseTool.new
 end
  

 
  
  if( not file_loaded?("KK_MortiseTool.rb"))
  
    # Access the main Draw menu
    draw_menu = UI.menu "Draw"
    # Add a separator and a submenu
    draw_menu.add_separator
    sub_menu = draw_menu.add_submenu("Tenon_Joint")
    # Add two menu items to the submenu
    it1 = sub_menu.add_item("M&T-Tenon") {tenonandmortise}
    it2 = sub_menu.add_item("LT-Mortise") {mortiseboth}
    it3 = sub_menu.add_item("Mortise") {addmortise}
    it4 = sub_menu.add_item("Tenon") {addtenon}
 
  end

 file_loaded("KK_MortiseTool.rb")
